"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
var _exportNames = {
  cleanup: true,
  render: true,
  fireEvent: true
};
exports.cleanup = cleanup;
exports.render = render;
exports.fireEvent = fireEvent;

var _testUtils = require("@vue/test-utils");

var _dom = require("@testing-library/dom");

Object.keys(_dom).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  if (Object.prototype.hasOwnProperty.call(_exportNames, key)) return;
  if (key in exports && exports[key] === _dom[key]) return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function get() {
      return _dom[key];
    }
  });
});

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

var mountedWrappers = new Set();

function render(Component) {
  var _mountOptions$global;

  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      {
    store = null,
    routes = null,
    container: customContainer,
    baseElement: customBaseElement
  } = _ref,
      mountOptions = _objectWithoutProperties(_ref, ["store", "routes", "container", "baseElement"]);

  var div = document.createElement('div');
  var baseElement = customBaseElement || customContainer || document.body;
  var container = customContainer || baseElement.appendChild(div);
  var plugins = ((_mountOptions$global = mountOptions.global) === null || _mountOptions$global === void 0 ? void 0 : _mountOptions$global.plugins) || [];

  if (store) {
    var {
      createStore
    } = require('vuex');

    plugins.push(createStore(store));
  }

  if (routes) {
    var requiredRouter = require('vue-router');

    var {
      createRouter,
      createWebHistory
    } = requiredRouter.default || requiredRouter;
    var routerPlugin = createRouter({
      history: createWebHistory(),
      routes
    });
    plugins.push(routerPlugin);
  }

  var wrapper = (0, _testUtils.mount)(Component, _objectSpread(_objectSpread({}, mountOptions), {}, {
    attachTo: container,
    global: _objectSpread(_objectSpread({}, mountOptions.global), {}, {
      plugins
    })
  })); // this removes the additional "data-v-app" div node from VTU:
  // https://github.com/vuejs/vue-test-utils-next/blob/master/src/mount.ts#L196-L213

  unwrapNode(wrapper.parentElement);
  mountedWrappers.add(wrapper);
  return _objectSpread({
    container,
    baseElement,
    debug: function debug() {
      var el = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : baseElement;
      var maxLength = arguments.length > 1 ? arguments[1] : undefined;
      var options = arguments.length > 2 ? arguments[2] : undefined;
      return Array.isArray(el) ? el.forEach(e => console.log((0, _dom.prettyDOM)(e, maxLength, options))) : console.log((0, _dom.prettyDOM)(el, maxLength, options));
    },
    unmount: () => wrapper.unmount(),
    html: () => wrapper.html(),
    emitted: () => wrapper.emitted(),
    rerender: props => wrapper.setProps(props)
  }, (0, _dom.getQueriesForElement)(baseElement));
}

function unwrapNode(node) {
  node.replaceWith(...node.childNodes);
}

function cleanup() {
  mountedWrappers.forEach(cleanupAtWrapper);
}

function cleanupAtWrapper(wrapper) {
  if (wrapper.element.parentNode && wrapper.element.parentNode.parentNode === document.body) {
    document.body.removeChild(wrapper.element.parentNode);
  }

  wrapper.unmount();
  mountedWrappers.delete(wrapper);
} // Vue Testing Library's version of fireEvent will call DOM Testing Library's
// version of fireEvent plus wait for one tick of the event loop to allow Vue
// to asynchronously handle the event.
// More info: https://vuejs.org/v2/guide/reactivity.html#Async-Update-Queue


function fireEvent() {
  return _fireEvent.apply(this, arguments);
}

function _fireEvent() {
  _fireEvent = _asyncToGenerator(function* () {
    (0, _dom.fireEvent)(...arguments);
    yield (0, _dom.waitFor)(() => {});
  });
  return _fireEvent.apply(this, arguments);
}

function suggestUpdateIfNecessary(eventValue, eventKey) {
  var changeOrInputEventCalledDirectly = eventValue && (eventKey === 'change' || eventKey === 'input');

  if (changeOrInputEventCalledDirectly) {
    console.warn("Using fireEvent.".concat(eventKey, "() may lead to unexpected results. Please use fireEvent.update() instead."));
  }
}

Object.keys(_dom.fireEvent).forEach(key => {
  fireEvent[key] = /*#__PURE__*/_asyncToGenerator(function* () {
    suggestUpdateIfNecessary(arguments.length <= 1 ? undefined : arguments[1], key);

    _dom.fireEvent[key](...arguments);

    yield (0, _dom.waitFor)(() => {});
  });
});

fireEvent.touch = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator(function* (elem) {
    yield fireEvent.focus(elem);
    yield fireEvent.blur(elem);
  });

  return function (_x) {
    return _ref3.apply(this, arguments);
  };
}(); // Small utility to provide a better experience when working with v-model.
// Related upstream issue: https://github.com/vuejs/vue-test-utils/issues/345#issuecomment-380588199
// Examples: https://github.com/testing-library/vue-testing-library/blob/master/src/__tests__/form.js


fireEvent.update = (elem, value) => {
  var tagName = elem.tagName;
  var type = elem.type;

  switch (tagName) {
    case 'OPTION':
      {
        elem.selected = true;
        var parentSelectElement = elem.parentElement.tagName === 'OPTGROUP' ? elem.parentElement.parentElement : elem.parentElement;
        return fireEvent.change(parentSelectElement);
      }

    case 'INPUT':
      {
        if (['checkbox', 'radio'].includes(type)) {
          elem.checked = true;
          return fireEvent.change(elem);
        } else if (type === 'file') {
          return fireEvent.change(elem);
        } else {
          elem.value = value;
          return fireEvent.input(elem);
        }
      }

    case 'TEXTAREA':
      {
        elem.value = value;
        return fireEvent.input(elem);
      }

    case 'SELECT':
      {
        elem.value = value;
        return fireEvent.change(elem);
      }

    default: // do nothing

  }

  return null;
}; // If we're running in a test runner that supports afterEach then we'll
// automatically run cleanup after each test. This ensures that tests run in
// isolation from each other.
// If you don't like this, set the VTL_SKIP_AUTO_CLEANUP variable to 'true'.


if (typeof afterEach === 'function' && !process.env.VTL_SKIP_AUTO_CLEANUP) {
  afterEach(() => {
    cleanup();
  });
}